import 'package:json_annotation/json_annotation.dart';
import 'package:intl/intl.dart' as Intl;
import 'dart:ui';

export 'package:json_annotation/json_annotation.dart';

/// Created by Luis on 09/02/19.


class AdeptDateTime
{
  static const String FULL_DATE_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss.SSS";
  static const String SIMPLE_DATE_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss";
  static const String FORMATTED_DATE_TIME_FORMAT = "EEE, d MMM yyyy hh:mm a";
  static const String FORMATTED_DATE_FORMAT = "EEE, d MMM yyyy";
  static const String FORMATTED_TIME_FORMAT = "hh:mm a";

  @JsonKey(ignore: true) int _dateTimeMillis;
  @JsonKey(ignore: true) DateTime _dateAsDateTime;
  @JsonKey(ignore: true) DateTime _dateAsDateTimeLocal;
  @JsonKey(ignore: true) String _dateFormat = SIMPLE_DATE_TIME_FORMAT;
  @JsonKey(ignore: true) String _formattedDateTimeFormat = FORMATTED_DATE_TIME_FORMAT;
  @JsonKey(ignore: true) String _formattedDateFormat = FORMATTED_DATE_FORMAT;
  @JsonKey(ignore: true) String _formattedTimeFormat = FORMATTED_TIME_FORMAT;
  String _date;

  AdeptDateTime({
    String date,
    int dateTimeMillis,
    DateTime dateAsDateTime,
    String dateFormatStandard,
    String formattedDateTimeFormat,
    String formattedDateFormat,
    String formattedTimeFormat,
  }) {
    _dateFormat = dateFormatStandard ?? SIMPLE_DATE_TIME_FORMAT;
    _formattedDateTimeFormat = formattedDateTimeFormat ?? FORMATTED_DATE_TIME_FORMAT;
    _formattedDateFormat = formattedDateFormat ?? FORMATTED_DATE_FORMAT;
    _formattedTimeFormat = formattedTimeFormat ?? FORMATTED_TIME_FORMAT;
    this.date = date;
    if(date == null){
      if(dateAsDateTime != null) this.dateAsDateTime = dateAsDateTime;
      if(dateTimeMillis != null) this.dateTimeMillis = dateTimeMillis;
    }
  }

  String format({String format}) {
    try{
      return Intl.DateFormat(format ?? formattedDateTimeFormat, window.locale.toString()).format(dateAsDateTimeLocal);
    }catch(e){
      print(e);
      try{
        return Intl.DateFormat(format ?? formattedDateTimeFormat).format(dateAsDateTimeLocal);
      }catch(e){
        print(e);
        return Intl.DateFormat().format(dateAsDateTimeLocal);
      }
    }
  }

  String formatted() => format(format: formattedDateTimeFormat);

  String formattedDate() => format(format: formattedDateFormat);

  String formattedTime() => format(format: formattedTimeFormat);

  int get dateTimeMillis => _dateTimeMillis ?? (_dateTimeMillis = dateAsDateTime.millisecondsSinceEpoch);
  set dateTimeMillis(int value) {
    _dateTimeMillis = value ?? _dateTimeMillis;
    this.dateAsDateTime = DateTime.fromMillisecondsSinceEpoch(dateTimeMillis);
  }

  DateTime get dateAsDateTime {
    if(_dateAsDateTime == null){
      if(_date != null) {
        try{
          _dateAsDateTime = Intl.DateFormat(dateFormat).parse(_date);
        }catch(e){
          print(e);
          _dateAsDateTime = DateTime.tryParse(_date);
        }
      }
      if(_dateAsDateTime == null)
        _dateAsDateTime = DateTime.now();
    }
    return _dateAsDateTime;
  }

  set dateAsDateTime(DateTime value) {
    _dateAsDateTime = value ?? _dateAsDateTime;
    if(dateAsDateTime != null){
      _dateAsDateTimeLocal = DateTime.fromMillisecondsSinceEpoch(_dateAsDateTime.millisecondsSinceEpoch);
      date = format(format: dateFormat);
    }
  }

  DateTime get dateAsDateTimeLocal => _dateAsDateTimeLocal ?? (_dateAsDateTimeLocal = dateAsDateTime);
  set dateAsDateTimeLocal(DateTime value) => _dateAsDateTimeLocal = value;

  String get dateFormat => _dateFormat ?? (_dateFormat = SIMPLE_DATE_TIME_FORMAT);
  set dateFormat(String value) => _dateFormat = value ?? _dateFormat;

  String get formattedDateTimeFormat => _formattedDateTimeFormat ?? (_formattedDateTimeFormat = FORMATTED_DATE_TIME_FORMAT);
  set formattedDateTimeFormat(String value) => _formattedDateTimeFormat = value ?? FORMATTED_DATE_TIME_FORMAT;

  String get formattedDateFormat => _formattedDateFormat ?? (_formattedDateFormat = FORMATTED_DATE_FORMAT);
  set formattedDateFormat(String value) => _formattedDateFormat = value ?? FORMATTED_DATE_FORMAT;

  String get formattedTimeFormat => _formattedTimeFormat ?? (_formattedTimeFormat = FORMATTED_TIME_FORMAT);
  set formattedTimeFormat(String value) => _formattedTimeFormat = value ?? FORMATTED_TIME_FORMAT;

  String get date {
    if(_date == null) dateAsDateTime;///this generates a non null date
    return _date;
  }
  set date(String value) => this._date = value ?? _date;

  String toJson() => _date;
  Map<String, dynamic> toJsonMap() => <String, dynamic>{'date': _date};
  factory AdeptDateTime.fromJson(String json) => json != null ? AdeptDateTime(date: json) : null;
  factory AdeptDateTime.fromJsonMap(Map<String, dynamic> json) => json != null ? AdeptDateTime(date: json['date'] as String) : null;
}